#!/usr/bin/python3
import os
import socket
import sys
import ssl
import pika
import threading
import logging
import time
import signal
import configparser
import argparse


def ping():  # respond to IRC server Pings.
    ircsock.send(bytes("PONG :pingis\n", "UTF-8"))


# sends messages to the IRC channel.
def sendmsg(msg, status=None, nick_to_alert=None):
    try:
        ircsock.send(bytes("PRIVMSG " + ircchannel + " :" +
                           msg + "\n", "UTF-8"))
        if status == 'FAILURE':
            failure_msg = 'please check this scalebot-jenkins-failure ^'
            if not nick_to_alert:
                ircsock.send(bytes("PRIVMSG " + ircchannel + " :" +
                                   failure_msg + "\n", "UTF-8"))
            else:
                ircsock.send(bytes("PRIVMSG " + ircchannel + " :"
                                   + nick_to_alert + ': ' + failure_msg +
                                   "\n", "UTF-8"))
        logging.debug('IRC: send msg: ' + str(msg))
    except:
        logging.error('IRC: Could not send msg: \"' + str(msg) +
                      '\" to channel: ' + str(ircchannel))


def joinchan(chan):  # join IRC channel
    ircsock.send(bytes("JOIN " + chan + "\n", "UTF-8"))
    ircmsg = ""
    print('> joinned IRC channel? ')
    while ircmsg.find("End of /NAMES list.") == -1:
        ircmsg = ircsock.recv(2048).decode("UTF-8")
        ircmsg = ircmsg.strip('\n\r')
        print(ircmsg)


def helpmsg(name):
    message = name + ': Hi! I am an IRC BOT. '\
           + 'I listen scalebot/jenkins messages sent to RabbitMQ-server '\
           + 'and I show them here.'
    sendmsg(message)


# Convert time received from Jenkins and RabbitMQ in miliseconds(ms)
def converttime(mseconds):
    seconds = (mseconds/1000) % 60
    minutes = (mseconds/(1000*60)) % 60
    hours = (mseconds/(1000*60*60)) % 24
    if hours >= 1:
        timetoshow = str(format(hours, '.2f'))+'h'
    else:
        timetoshow = str(int(minutes)) + '\'' + str(int(seconds)) + '\"'
    return timetoshow


def run_rabbit_consumer(nick_to_alert):  # RabbitMQ-server consumer
    print('RabbitMQ consumer...')

    # Check body messages, ignoring queued and dequeued messages
    def callback(channel, method, properties, body):
        print(" [x] Received %r" % body)
        msg_to_sent = str(body.decode("UTF-8"))
        msg_dict = eval(msg_to_sent)
        if msg_dict['state'] == 'STARTED':
            msg_to_sent = 'Starting job: ' + msg_dict['build_job_name'] \
                          + ' Build #' + str(msg_dict['build_number'])
            sendmsg(msg_to_sent)
        if msg_dict['state'] == 'COMPLETED':
            # time in ms
            timetoshow = converttime(int(msg_dict['build_duration']))
            msg_to_sent = 'Finished job: ' + msg_dict['build_job_name'] + \
                          ' Build #' + str(msg_dict['build_number']) + \
                          ': ' + msg_dict['status'] + ' in ' + \
                          str(timetoshow) + ': ' + \
                          msg_dict['url']+'console'
            sendmsg(msg_to_sent, msg_dict['status'],
                    nick_to_alert=nick_to_alert)

    channel.basic_consume(callback,
                          queue='scalebot',
                          no_ack=True)

    logging.info('RabbitMQ consumer started')
    print(' [*] Waiting for messages. To exit press CTRL+C')
    channel.start_consuming()


def scalebot_ircbot(botnick):
    print('main irc')
    while 1:
        print("inf loop")
        ircmsg = ""
        ircmsg = ircsock.recv(2048).decode("UTF-8")
        # remove any line breaks
        ircmsg = str(ircmsg.strip('\n\r'))
        print('here print ircmsg: ', ircmsg)

        # repsond to PINGs, so server doesn't think we've disconnected
        if ircmsg.find("PING :") != -1:
            ping()

        # look for PRIVMSG lines as these are messages in the channel or
        # sent to the bot
        if ircmsg.find("PRIVMSG") != -1:
            # save user name into name variable
            name = ircmsg.split('!', 1)[0][1:]
            print('name: ' + name)
            logging.info('IRC: msg: ' + ircmsg)

            # get the message to look for commands
            message = ircmsg.split('PRIVMSG', 1)[1].split(':', 1)[1]
            print(message)
            # look for commands and send to appropriate function.
            if (botnick in message) and ('help' in message):
                helpmsg(name)
            elif message.startswith('help') or message.startswith('!help'):
                helpmsg(name)
            elif message == 'oi':  # answering 'oi'(Hi) msg - a manual ping.
                sendmsg('oi')


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--config',
                        help=("Name of the .conf file " +
                              "from which to read configurations"),
                        required=True, type=str)
    args = parser.parse_args()
    if (len(sys.argv)) < 2:
        print('missing config file, add parameter -c or ' +
              '--config file_name.conf')
        logging.info('Missing .conf file from which to read')
        parser.print_help()
        sys.exit(1)
    logfile = os.environ['HOME']+'/irc_rabbit.log'  # LOG file name
    print(logfile)
    logging.basicConfig(filename=logfile, level=logging.DEBUG,
                        format='%(asctime)s %(levelname)s: %(message)s')

    config = configparser.ConfigParser()
    config.readfp(open(args.config))
    rabbitmq_IP = config.get('rabbitmq', 'rabbitmq_IP')
    rabbitmq_user = config.get('rabbitmq', 'rabbitmq_user')
    rabbitmq_pass = config.get('rabbitmq', 'rabbitmq_pass')
    rabbitmq_port = config.getint('rabbitmq', 'rabbitmq_port')
    rabbitmq_vhost = config.get('rabbitmq', 'rabbitmq_vhost')

    credentials = pika.PlainCredentials(rabbitmq_user, rabbitmq_pass)

    parameters = pika.ConnectionParameters(rabbitmq_IP, rabbitmq_port,
                                           rabbitmq_vhost, credentials)

    try:
        connection = pika.BlockingConnection(parameters)
        print("connection - rabbitmq: ", connection)
        channel = connection.channel()
        print('channel rabbitmq: ', channel)
        logging.info('RabbitMQ-server: connected')
    except:
        logging.error('Could not connect to RabbitMQ server')
        print(' > Could not connect to RabbitMQ server, exiting')
        sys.exit(1)

    ircserver = config.get('ircbot', 'irc_server')
    print('ircserver: ', ircserver)

    ircpasswd = config.get('ircbot', 'irc_password')
    botnick = config.get('ircbot', 'irc_botnick')
    nick_to_alert = config.get('ircbot', 'irc_nick_to_alert')
    ircchannel = '#'+config.get('ircbot', 'irc_channel')
    print('ircchannel: ', ircchannel)
    # SSL: 6697 _or_ regular 6667 if not SSL
    port = config.getint('ircbot', 'irc_port')
    context = ssl.create_default_context()
    print('ssl conte', context)
    ircsock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    print(ircsock)
    irc_enable_ssl = config.getboolean('ircbot', 'irc_enable_ssl')
    ircauth = config.get('ircbot', 'irc_auth_method')
    if irc_enable_ssl:
        ircsock = context.wrap_socket(ircsock, server_hostname=ircserver)
        print('IRC: enable_ssl: True ')

    try:
        ircsock.connect((ircserver, port))
        print('ircsock.connect')

        if ircauth == 'server':
            # auth_method: server
            ircsock.send(bytes("PASS " + ircpasswd + "\n", "UTF-8"))
            print('IRC: authentication method: server via password ')
            logging.info('IRC: authentication method: server via password ')

        print('after ...~')
        # assign the nick to the bot
        ircsock.send(bytes("NICK " + botnick + "\n", "UTF-8"))
        print('after nick')
        # Filling out a form with this line and saying to
        # set all the fields to the bot nickname:
        ircsock.send(bytes("USER " + botnick + " " + botnick + " " +
                           botnick + " " + botnick + "\n", "UTF-8"))
        print('after user')

        if ircauth == 'nickserv':
            ircsock.send(bytes("nickserv identify " + ircpasswd + "\r\n",
                               "UTF-8"))
            logging.info('IRC: authentication method: nickserv')

        logging.info('IRC: connected')
    except:
        logging.error('Could not connect to IRC server: ' + ircserver)
        print(' > Could not connect to IRC server: ',
              ircserver + ' exiting...')
        sys.exit(1)

    time.sleep(30)
    joinchan(ircchannel)  # joining the channel.
    logging.info('IRC: join channel: ' + ircchannel + '\n')
    print('--------------------------------------------------------------')
    th_rmq = threading.Thread(target=run_rabbit_consumer, args=[nick_to_alert])
    th_irc = threading.Thread(target=scalebot_ircbot, args=[botnick])

    try:
        th_rmq.start()
        th_irc.start()
        signal.pause()
    except (KeyboardInterrupt, SystemExit):
        print('stopping .. ')
        logging.debug('closing connections')
        ircsock.close()
        connection.close()
