from charms.reactive import (
 hook, when, when_not, when_any, set_flag, set_state, remove_state )
from charmhelpers.core import hookenv
from charmhelpers.core.hookenv import (
 log, status_set, relation_get, ingress_address, relation_ids, related_units )
import os, subprocess, configparser
from relations.rabbitmq.requires import RabbitMQRequires


@hook('config-changed')
def config_changed():
    remove_state('ircbot.all_config')
    
@when_not('amqp.connected')
def blocked():
    status_set('blocked', 'Waiting for AMQP relation')

@hook('amqp-relation-joined')
def create_user(relation_id=None):

    config = hookenv.config()

    rabbitmq_user = config.get('rabbitmq_user')
    rabbitmq_vhost = config.get('rabbitmq_vhost')
    
    interface_rabbitmq = RabbitMQRequires(relation_name='amqp')
    interface_rabbitmq.request_access(username=rabbitmq_user, vhost=rabbitmq_vhost)
    
    rg = relation_get()
    log("@amqp-relation-joined hook : %s" %rg, level="INFO")

@hook('amqp-relation-changed')
def changed(relation_id=None):
    status_set('maintenance', 'AMQP relation changed')
    rids = relation_ids('amqp')
    log("@amqp-relation-changed Rel-id: '%s'" %rids, level="INFO") 

    rg = relation_get()
    log("@amqp-relation-changed hook: '%s'" %rg, level="INFO")

    remove_state('ircbot.all_config')
    set_state('amqp.configured')

@hook('amqp-relation-broken')
def broken(relation_id=None):
    status_set('blocked', 'AMQP relation broken')
    remove_state('ircbot.all_config')

@hook('amqp-relation-departed')
def departed(relation_id=None):
    status_set('blocked', 'AMQP relation departed')
    remove_state('ircbot.all_config')

@when_not('amqp-relation-broken')
@when_not('amqp-relation-departed')
@when('amqp.configured')
@when_any('amqp.available' , 'config.changed')
def create_conf_file():

    log (' *** Creating .conf file')

#get configuration from bundle.yaml or from --config
    config = hookenv.config()

    rabbitmq_user = config.get('rabbitmq_user')
    rabbitmq_port = config.get('rabbitmq_port')
    rabbitmq_vhost = config.get('rabbitmq_vhost')

    ircserver = config.get('irc_server')
    ircchannel = str(config.get('irc_channel'))
    ircport = config.get('irc_port') 
    ircbotnick = config.get('irc_botnick')
    ircssl = config.get('irc_enable_ssl')
    ircpassword = config.get('irc_password')
    ircauth = config.get('irc_auth_method')
    ircalert = config.get('irc_nick_to_alert')

#add configuration to a .conf file: 
#files/ircbot.py will read from it
    
    confile = configparser.RawConfigParser()
    confile.add_section('ircbot')
    confile.add_section('rabbitmq')

    confile.set('rabbitmq', 'rabbitmq_user', rabbitmq_user)

    ctxt = {}
    for rid in relation_ids('amqp'):
        for unit in related_units(rid):
            ctxt.update({'rabbitmq_user': rabbitmq_user,
                         'rabbitmq_password': relation_get('password', rid=rid, unit=unit)
            })
            rabbitmq_IP = ingress_address(rid=rid, unit=unit)
            log ('*** Rabbitmq-server IP address: {}'.format(rabbitmq_IP), level='INFO')
            confile.set('rabbitmq', 'rabbitmq_IP', rabbitmq_IP)

            rabbitmq_pass = ctxt['rabbitmq_password']
            confile.set('rabbitmq', 'rabbitmq_pass', rabbitmq_pass)
    log("@create_conf_file:  updated config {}".format(ctxt), level="INFO")

    confile.set('rabbitmq', 'rabbitmq_port', rabbitmq_port)
    confile.set('rabbitmq', 'rabbitmq_vhost', rabbitmq_vhost)
    confile.set('ircbot', 'irc_server', ircserver)
    confile.set('ircbot', 'irc_channel', ircchannel) 
    confile.set('ircbot', 'irc_botnick', ircbotnick)
    confile.set('ircbot', 'irc_port', ircport)
    confile.set('ircbot', 'irc_enable_ssl', ircssl)
    confile.set('ircbot', 'irc_password',  ircpassword) 
    confile.set('ircbot', 'irc_auth_method', ircauth)
    confile.set('ircbot', 'irc_nick_to_alert', ircalert)

    confile_path = os.path.join(hookenv.charm_dir(), 'files', 'irc_rabbit.conf')

    with open(confile_path, 'w') as c:
         confile.write(c)
    set_state('ircbot.conf_file')
    set_state('ircbot.all_config')

@when('ircbot.conf_file')
@when_not('ircbot.created.systemd.service')
def create_systemd_service():
    
    log('@creating systemd unit file ')
    confserv = configparser.ConfigParser()
    confserv_path = os.path.join(hookenv.charm_dir(), 'files', 'ircbot.service')
    confserv.optionxform = lambda option: option
    confserv.read(confserv_path)
    file_to_exec = os.path.join(hookenv.charm_dir(), 'files', 'ircbot.py')
    confile_path = os.path.join(hookenv.charm_dir(), 'files', 'irc_rabbit.conf')
    exec_path = '/bin/bash -c "sudo python3 '+ file_to_exec + ' -c ' + confile_path + '"'
    confserv.set('Service', 'ExecStart', exec_path)

    with open(confserv_path, 'w') as c:
         confserv.write(c, space_around_delimiters=False)

    set_flag('ircbot.created.systemd.service')

@when_not('ircbot.service.at.etc.dir')
@when('ircbot.created.systemd.service')
def copy_to_etc_dir():

    log('@copying ircbot.service file to /etc/systemd/system/')
    confserv_path = os.path.join(hookenv.charm_dir(), 'files', 'ircbot.service')
    subprocess.run(['cp', confserv_path, '/etc/systemd/system/'])
    subprocess.run(['systemctl', 'daemon-reload'], check=True)
    subprocess.run(['systemctl', 'enable', 'ircbot'], check=True)

    set_flag('ircbot.service.at.etc.dir')

@when('ircbot.all_config')
@when('amqp.available')
@when('ircbot.service.at.etc.dir')
def run_ircbot():
    try:
        subprocess.run(['systemctl', 'start', 'ircbot'], check=True)
        log ('---- Running  IRCbot.service ---- ')
        status_set('active', 'Running IRCbot')
    except:
        status_set('blocked', 'problem to run IRCbot, check config')

@when('ircbot.service.at.etc.dir')
@when_not('ircbot.all_config')
def stop_ircbot():
    subprocess.run(['systemctl', 'stop', 'ircbot'], check=True)
    log("@STOPPING IRCbot.service, wait...", level="INFO")
    status_set('maintenance', 'stopped IRCbot.service')

@hook('update-status')
def update():
    status_set('active', 'ready')
